@@ -325,8 +325,11 @@ void MediaObject::connectAudio(GstPad *pad)
  *
  * returns true if successful
  */
-bool MediaObject::createPipefromURL(const QString &url)
+bool MediaObject::createPipefromURL(const QString &encodedUrl)
 {
+    // Convert back to URL
+    QUrl url(encodedUrl, QUrl::StrictMode);
+
     // Remove any existing data source
     if (m_datasource) {
         gst_bin_remove(GST_BIN(m_pipeline), m_datasource);
@@ -335,13 +338,14 @@ bool MediaObject::createPipefromURL(const QString &url)
     }
 
     // Verify that the uri can be parsed
-    if (!gst_uri_is_valid(qPrintable(url))) {
-        m_backend->logMessage(QString("%0 is not a valid URI").arg(url));
+    if (!url.isValid()) {
+        m_backend->logMessage(QString("%1 is not a valid URI").arg(encodedUrl));
         return false;
     }
 
     // Create a new datasource based on the input URL
-    m_datasource = gst_element_make_from_uri(GST_URI_SRC, qPrintable(url), NULL);
+    QByteArray encoded_cstr_url = url.toEncoded();
+    m_datasource = gst_element_make_from_uri(GST_URI_SRC, encoded_cstr_url.constData(), NULL);
     if (!m_datasource)
         return false;
 
@@ -373,7 +377,7 @@ bool MediaObject::createPipefromStream(const MediaSource &source)
         return false;
 
     StreamReader *streamReader = new StreamReader(source);
-    g_object_set (G_OBJECT (m_datasource), "iodevice", streamReader, NULL);
+    g_object_set (G_OBJECT (m_datasource), "iodevice", streamReader, (const char*)NULL);
 
     // Link data source into pipeline
     gst_bin_add(GST_BIN(m_pipeline), m_datasource);
@@ -409,7 +413,7 @@ void MediaObject::createPipeline()
     // pull-mode access. Also note that the max-size-time are increased to
     // reduce buffer overruns as these are not gracefully handled at the moment.
     m_audioPipe = gst_element_factory_make("queue", NULL);
-    g_object_set(G_OBJECT(m_audioPipe), "max-size-time",  MAX_QUEUE_TIME, NULL);
+    g_object_set(G_OBJECT(m_audioPipe), "max-size-time",  MAX_QUEUE_TIME, (const char*)NULL);
     gst_bin_add(GST_BIN(m_audioGraph), m_audioPipe);
     GstPad *audiopad = gst_element_get_pad (m_audioPipe, "sink");
     gst_element_add_pad (m_audioGraph, gst_ghost_pad_new ("sink", audiopad));
@@ -421,7 +425,7 @@ void MediaObject::createPipeline()
     gst_object_sink (GST_OBJECT (m_videoGraph));
 
     m_videoPipe = gst_element_factory_make("queue", NULL);
-    g_object_set(G_OBJECT(m_videoPipe), "max-size-time", MAX_QUEUE_TIME, NULL);
+    g_object_set(G_OBJECT(m_videoPipe), "max-size-time", MAX_QUEUE_TIME, (const char*)NULL);
     gst_bin_add(GST_BIN(m_videoGraph), m_videoPipe);
     GstPad *videopad = gst_element_get_pad (m_videoPipe, "sink");
     gst_element_add_pad (m_videoGraph, gst_ghost_pad_new ("sink", videopad));
@@ -843,21 +847,11 @@ void MediaObject::setSource(const MediaSource &source)
     m_metaData.clear();
 
     switch (source.type()) {
-    case MediaSource::Url: {
-            QString urlString = source.url().toString();
-            if (!createPipefromURL(urlString)) {
-                setError(tr("Could not open media source."));
-                return;
-            }
-        }
-        break;
-
-    case MediaSource::LocalFile: {
-            QString urlString = QUrl::fromLocalFile(source.fileName()).toString();
-            if (!createPipefromURL(urlString)) {
-                setError(tr("Could not open media source."));
-                return;
-            }
+    case MediaSource::Url:
+    case MediaSource::LocalFile:
+        if (!createPipefromURL(source.url().toString())) {
+            setError(tr("Could not open media source."));
+            return;
         }
         break;
 
@@ -1113,9 +1107,9 @@ void MediaObject::handleBusMessage(const Message &message)
     if (m_backend->debugLevel() >= Backend::Debug) {
         int type = GST_MESSAGE_TYPE(gstMessage);
         gchar* name = gst_element_get_name(gstMessage->src);
-        QString message = QString("Bus: %0 (%1)").arg(gst_message_type_get_name ((GstMessageType)type)).arg(name);
+        QString msgString = QString("Bus: %0 (%1)").arg(gst_message_type_get_name ((GstMessageType)type)).arg(name);
         g_free(name);
-        m_backend->logMessage(message, Backend::Debug, this);
+        m_backend->logMessage(msgString, Backend::Debug, this);
     }
 
     switch (GST_MESSAGE_TYPE (gstMessage)) {
@@ -1204,7 +1198,6 @@ void MediaObject::handleBusMessage(const Message &message)
             QString logMessage;
             gst_message_parse_error (gstMessage, &err, &debug);
             gchar *errorMessage = gst_error_get_message (err->domain, err->code);
-            QString message = QString::fromUtf8(errorMessage);
             logMessage.sprintf("Error: %s Message:%s (%s) Code:%d", debug, err->message, errorMessage, err->code);
             m_backend->logMessage(logMessage, Backend::Warning);
             g_free(errorMessage);
@@ -1252,9 +1245,9 @@ void MediaObject::handleBusMessage(const Message &message)
             gchar *debug;
             GError *err;
             gst_message_parse_warning(gstMessage, &err, &debug);
-            QString message;
-            message.sprintf("Warning: %s\nMessage:%s", debug, err->message);
-            m_backend->logMessage(message, Backend::Warning);
+            QString msgString;
+            msgString.sprintf("Warning: %s\nMessage:%s", debug, err->message);
+            m_backend->logMessage(msgString, Backend::Warning);
             g_free (debug);
             g_error_free (err);
             break;
